//------------------------------------------------------------------------------
//title
//	[^vO
//ver
//	2.0
//designer
//	igeta kenichiro
//------------------------------------------------------------------------------

#include <machine.h>
#include <_h_c_lib.h>
#include "iodefine.h"

#define MPU_CLK			20000000UL					//}CRg[Hz]
#define MPU_TIME		(1000000000L/MPU_CLK)		//}CR쎞[ns]
#define TIME_BASE		1000000L					//[ns]
#define ENCODER_PULSE	1000						//GR[_pX
#define CONVERT_BASE	(MPU_CLK/ENCODER_PULSE*60)	//x=CONVERT_BASE/pX
#define LOG_SIZE		160							//f[^
#define SPEED_LOOP		1							//x[ms]
#define SLOW_UP_LOOP	1							//X[Abv_E[ms]
#define PGAIN			4000						//QC
#define IGAIN			25							//ϕQC

#define SW_STOP			IO.PDRB.BIT.B2
#define SW_MODE			IO.PDRB.BIT.B3
#define SW_SLOWUP		IO.PDRB.BIT.B4
#define SW_PGAIN		IO.PDRB.BIT.B5
#define SW_IGAIN		IO.PDRB.BIT.B6
#define SW_DIR			IO.PDRB.BIT.B7

//------------------------------------------------------------------------------

void SlowUpDown(void);
void MotorOut(long out, int dir);
void SpeedLoop(void);
void initialize(void);
void Sleep(unsigned long time);
void main(void);

//------------------------------------------------------------------------------

volatile unsigned long gSystemTime;

volatile unsigned int gWidthA;				//pX(A)
volatile int gSetSpeed;				//ݒ葬x
volatile int gSet2Speed;			//ݒ葬x
volatile int gProportionalGain;		//QC
volatile int gIntegralGain;			//ϕQC
volatile int gSlowUpRatio;			//X[Abvl

int gSpeedLog[LOG_SIZE];			//葬xobt@
int gSetSpeedLog[LOG_SIZE];			//ݒ葬xobt@
int gLogCount = LOG_SIZE;

//------------------------------------------------------------------------------
//	^C}[v
//------------------------------------------------------------------------------

__interrupt(vect=21)
void INT_TimerW(void)
{
	static int oldCapTime;
	static int flag;

	//pX
	if(TW.TSRW.BIT.IMFA){					//pXoH
		IO.PDR5.BIT.B2 = 0;					//LED2on
		TW.TSRW.BIT.IMFA = 0;
		if(flag == 0){
			gWidthA = TW.GRA - oldCapTime;	//pXvZ
		}
		oldCapTime = TW.GRA;
		TW.GRC = TW.GRA+0x8000;				//pXI[o[t[ݒ
		flag = 0;

	}
	//pX\͈͂𒴂Ăꍇ̏
	if(TW.TSRW.BIT.IMFC){					//pXI[o[t[H
		IO.PDR5.BIT.B2 = 1;					//LED2off
		TW.TSRW.BIT.IMFC = 0;
		TW.GRC = TW.GRC+0x8000;
		gWidthA = 0xffff;					//pXőɐݒ
		flag = 1;							//pXI[o[t[tOZbg

	}
	//ݏ
	if(TW.TSRW.BIT.IMFD){					//ݔH
		TW.TSRW.BIT.IMFD = 0;
		TW.GRD = TW.GRD+TIME_BASE/MPU_TIME;	//TIME_BASEԎŊ
		gSystemTime += TIME_BASE/1000000;
		if(gSystemTime%SPEED_LOOP==0){		//xH
			SpeedLoop();
		}
		if(gSystemTime%SLOW_UP_LOOP==0){	//X[Abv_EH
			SlowUpDown();
		}
	}
}

//------------------------------------------------------------------------------
//	[^hCoo
//------------------------------------------------------------------------------

void MotorOut(long out, int dir)
{
	TV.TCORB = out >> 12;					//ovlݒ

	if(dir >= 0){							//]H
		IO.PDR7.BIT.B4 = 1;
		IO.PDR7.BIT.B5 = 0;

	}else{									//]H
		IO.PDR7.BIT.B4 = 0;
		IO.PDR7.BIT.B5 = 1;
	}
}

//------------------------------------------------------------------------------
//	X[Abv_E
//------------------------------------------------------------------------------

void SlowUpDown(void)
{
	if(gSlowUpRatio != 0){				//X[to^cnvl[hH
		if(gSet2Speed < gSetSpeed){
			gSet2Speed+=gSlowUpRatio;
			if(gSet2Speed > gSetSpeed){
				gSet2Speed = gSetSpeed;
			}

		}else if(gSet2Speed > gSetSpeed){
			gSet2Speed-=gSlowUpRatio;
			if(gSet2Speed < gSetSpeed){
				gSet2Speed = gSetSpeed;
			}
		}
	}else{
		gSet2Speed = gSetSpeed;
	}

	if(gSet2Speed != gSetSpeed){
		IO.PDR5.BIT.B1 = 0;			//LED3on
	}else{
		IO.PDR5.BIT.B1 = 1;			//LED3off
	}
}

//------------------------------------------------------------------------------
//	x
//------------------------------------------------------------------------------

void SpeedLoop(void)
{
	static int count;
	static int speed;
	int setSpeed;
	int proportionalGain;
	int integralGain = 10;
	int difference;
	long proportional;
	static long integral;
	long out;

	setSpeed = gSet2Speed;

	proportionalGain = gProportionalGain;
	integralGain = gIntegralGain;

	//pXx(xtB[hobNM)ϊ
	if(gWidthA < 0x7fff){
		speed = CONVERT_BASE/gWidthA;	//pXȂƂxvZ

	}else{
		speed = 0;						//pXs\ȂƂx0
	}

	//΍vZ
	//΍=xw-xtB[hobNM
	if(setSpeed >= 0){
		difference = setSpeed-speed;	//]̂Ƃ
	}else{
		difference = -setSpeed-speed;	//t]̂Ƃ
	}

	//QCZ
	proportional = difference * proportionalGain;
	if(proportional > 0xfffff){		//̒llȏH
		proportional = 0xfffff;
	}else if(proportional < -0xfffff){	//̒llȉH
		proportional = 0;
	}

	//ϕQCZ
	integral = integral + difference * integralGain;
	if(integral > 0xfffff){		//̒llȏH
		integral = 0xfffff;
	}else if(integral < 0){			//̒llȉH
		integral = 0;
	}

	out = proportional+integral;
	if(out > 0xfffff){			//̒llȏH
		out = 0xfffff;
	}else if(out < 0){			//̒llȉH
		out = 0;
	}

	//[^[^hCoo
	MotorOut(out, setSpeed);

	//xf[^ۑ(sv)
	if(gLogCount < LOG_SIZE){
		if(count%2==0){
			gSpeedLog[gLogCount] = speed;
			gSetSpeedLog[gLogCount] = setSpeed;
			gLogCount++;
		}
		count++;
	}

}

//------------------------------------------------------------------------------
//	
//------------------------------------------------------------------------------

void initialize(void)
{
	IO.PDR1.BYTE = 0xf8;
	IO.PCR1      = 0xef;

	IO.PDR2.BYTE = 0xff;
	IO.PCR2      = 0xfd;

	IO.PDR5.BYTE = 0xff;
	IO.PCR5      = 0xff;

	IO.PDR7.BYTE = 0x8f;
	IO.PCR7      = 0xff;

	IO.PDR8.BYTE = 0xff;
	IO.PCR8      = 0xf9;

	//`^cݒ
	AD.ADCSR.BYTE = 0x11;
	AD.ADCSR.BIT.ADST = 1;

	//^C}[uݒ
	//PWMg=MPU_CLK/4/256
	TV.TCRV0.BYTE = 0x09;
	TV.TCRV1.BYTE = 0x00;
	TV.TCSRV.BYTE = 0x06;
	TV.TCORA      = 255;
	TV.TCORB      = 0;

	//^C}[vݒ
	TW.TMRW.BYTE  = 0x80;
	TW.TCRW.BYTE  = 0x00;
	TW.TIERW.BYTE = 0x0d;
	TW.TIOR0.BYTE = 0x44;
	TW.TIOR1.BYTE = 0x00;
	TW.GRC = 0x8000;
	TW.GRD = TIME_BASE/MPU_TIME;

	set_imask_ccr(0);				//݋

	//VA|[gݒ(sv)
	SCI3.SCR3.BYTE = 0x00;
	SCI3.SMR.BYTE = 0x00;
	SCI3.BRR = 15;
	IO.PMR1.BIT.TXD = 1;
	Sleep(10);
	SCI3.SCR3.BIT.TE = 1;
	SCI3.SCR3.BIT.RE = 1;
	SCI3.SCR3.BIT.RIE = 1;	//M݋
}

//------------------------------------------------------------------------------
//	ԑ҂[ms]
//------------------------------------------------------------------------------

void Sleep(unsigned long time)
{
	unsigned long oldTime = gSystemTime;

	while(gSystemTime-oldTime <= time){
		sleep();	//邱Ƃ̂Ń}CR~
	}
}

//------------------------------------------------------------------------------
//	C
//------------------------------------------------------------------------------

void main(void)
{
	int sw;

	initialize();

	while(1){
		if(SW_MODE == 1){
			while(IO.PDR1.BIT.B4 == 1){	//SW܂ő҂
				if(SW_MODE == 0){
					break;
				}
				Sleep(10);
			}
			IO.PDR5.BIT.B0 = 0;			//LED4on
			while(IO.PDR1.BIT.B4 == 0){	//SW܂ő҂
				Sleep(10);
			}
			IO.PDR5.BIT.B0 = 1;			//LED4off
			gLogCount = 0;				//sv
			Sleep(100);
		}

		if(SW_SLOWUP == 1){				//X[to^cnvl[hH
			gSlowUpRatio = AD.ADDRB/6553+1;
		}else{
			gSlowUpRatio = 0;
		}
		if(SW_STOP == 0){
			gSetSpeed = 0;

		}else if(SW_DIR == 0){
			gSetSpeed = AD.ADDRA/32;

		}else{
			gSetSpeed = -(AD.ADDRA/32);
		}
	
		if(SW_PGAIN == 0){
			gProportionalGain = PGAIN;	//QCW
		}else{
			gProportionalGain = PGAIN*2;//QCQ{
		}

		if(SW_IGAIN == 0){
			gIntegralGain = IGAIN;		//ϕQCW
		}else{
			gIntegralGain = IGAIN*2;	//ϕQCQ{
		}
		Sleep(1);
	}
}

//------------------------------------------------------------------------------
